@@ -74,13 +74,13 @@ Curl_inet_pton(int af, const char *src, void *dst)
 {
   switch (af) {
   case AF_INET:
-    return (inet_pton4(src, dst));
+    return (inet_pton4(src, (unsigned char *)dst));
 #ifdef ENABLE_IPV6
 #ifndef AF_INET6
-#define AF_INET6        AF_MAX+1        /* just to let this compile */
+#define AF_INET6        (AF_MAX+1)        /* just to let this compile */
 #endif
   case AF_INET6:
-    return (inet_pton6(src, dst));
+    return (inet_pton6(src, (unsigned char *)dst));
 #endif
   default:
     errno = EAFNOSUPPORT;
@@ -114,11 +114,11 @@ inet_pton4(const char *src, unsigned char *dst)
     const char *pch;
 
     if ((pch = strchr(digits, ch)) != NULL) {
-      u_int new = *tp * 10 + (pch - digits);
+      unsigned int val = *tp * 10 + (unsigned int)(pch - digits);
 
-      if (new > 255)
+      if (val > 255)
         return (0);
-      *tp = new;
+      *tp = val;
       if (! saw_digit) {
         if (++octets > 4)
           return (0);
@@ -161,7 +161,7 @@ inet_pton6(const char *src, unsigned char *dst)
   unsigned char tmp[IN6ADDRSZ], *tp, *endp, *colonp;
   const char *xdigits, *curtok;
   int ch, saw_xdigit;
-  u_int val;
+  unsigned int val;
 
   memset((tp = tmp), 0, IN6ADDRSZ);
   endp = tp + IN6ADDRSZ;
