@@ -1,16 +1,16 @@
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2004, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: telnet.c,v 1.61 2004/05/12 12:06:39 bagder Exp $
+ * $Id: telnet.c,v 1.78 2006-07-19 21:14:02 yangtse Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -30,10 +30,12 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
-
-#include <errno.h>
+#endif
 
 #if defined(WIN32)
 #include <time.h>
@@ -61,10 +63,6 @@
 #include <sys/param.h>
 #endif
 
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-
 
 #endif
 
@@ -73,6 +71,7 @@
 #include "transfer.h"
 #include "sendf.h"
 #include "telnet.h"
+#include "connect.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
@@ -82,6 +81,7 @@
 
 #include "arpa_telnet.h"
 #include "memory.h"
+#include "select.h"
 
 /* The last #include file should be: */
 #include "memdebug.h"
@@ -107,12 +107,12 @@ static CURLcode check_wsock2 ( struct SessionHandle *data );
 
 static
 void telrcv(struct connectdata *,
-	    unsigned char *inbuf,	/* Data received from socket */
-	    int count);			/* Number of bytes received */
+            unsigned char *inbuf,       /* Data received from socket */
+            ssize_t count);             /* Number of bytes received */
 
 static void printoption(struct SessionHandle *data,
-			const char *direction,
-			int cmd, int option);
+                        const char *direction,
+                        int cmd, int option);
 
 static void negotiate(struct connectdata *);
 static void send_negotiation(struct connectdata *, int cmd, int option);
@@ -120,16 +120,17 @@ static void set_local_option(struct connectdata *, int cmd, int option);
 static void set_remote_option(struct connectdata *, int cmd, int option);
 
 static void printsub(struct SessionHandle *data,
-		     int direction, unsigned char *pointer, int length);
+                     int direction, unsigned char *pointer,
+                     size_t length);
 static void suboption(struct connectdata *);
 
 /* For negotiation compliant to RFC 1143 */
-#define CURL_NO	         0
-#define CURL_YES 	 1
-#define CURL_WANTYES	 2
-#define CURL_WANTNO	 3
+#define CURL_NO          0
+#define CURL_YES         1
+#define CURL_WANTYES     2
+#define CURL_WANTNO      3
 
-#define CURL_EMPTY	 0
+#define CURL_EMPTY       0
 #define CURL_OPPOSITE    1
 
 /*
@@ -151,20 +152,20 @@ typedef enum
 struct TELNET {
   int please_negotiate;
   int already_negotiated;
-  int us[256]; 
-  int usq[256]; 
-  int us_preferred[256]; 
-  int him[256]; 
-  int himq[256]; 
-  int him_preferred[256]; 
+  int us[256];
+  int usq[256];
+  int us_preferred[256];
+  int him[256];
+  int himq[256];
+  int him_preferred[256];
   char subopt_ttype[32];             /* Set with suboption TTYPE */
   char subopt_xdisploc[128];          /* Set with suboption XDISPLOC */
   struct curl_slist *telnet_vars; /* Environment variables */
 
   /* suboptions */
-  char subbuffer[SUBBUFSIZE];
-  char *subpointer, *subend;      /* buffer for sub-options */
-  
+  unsigned char subbuffer[SUBBUFSIZE];
+  unsigned char *subpointer, *subend;      /* buffer for sub-options */
+
   TelnetReceive telrcv_state;
 };
 
@@ -172,22 +173,22 @@ struct TELNET {
 static CURLcode
 check_wsock2 ( struct SessionHandle *data )
 {
-  int err; 
-  WORD wVersionRequested;  
-  WSADATA wsaData; 
+  int err;
+  WORD wVersionRequested;
+  WSADATA wsaData;
 
   curlassert(data);
 
   /* telnet requires at least WinSock 2.0 so ask for it. */
   wVersionRequested = MAKEWORD(2, 0);
 
-  err = WSAStartup(wVersionRequested, &wsaData); 
-  
+  err = WSAStartup(wVersionRequested, &wsaData);
+
   /* We must've called this once already, so this call */
   /* should always succeed.  But, just in case... */
   if (err != 0) {
     failf(data,"WSAStartup failed (%d)",err);
-    return CURLE_FAILED_INIT; 
+    return CURLE_FAILED_INIT;
   }
 
   /* We have to have a WSACleanup call for every successful */
@@ -199,7 +200,7 @@ check_wsock2 ( struct SessionHandle *data )
       HIBYTE(wsaData.wVersion) != HIBYTE(wVersionRequested)) {
       /* Our version isn't supported */
       failf(data,"insufficient winsock version to support "
-	    "telnet");
+            "telnet");
       return CURLE_FAILED_INIT;
   }
 
@@ -212,29 +213,17 @@ CURLcode init_telnet(struct connectdata *conn)
 {
   struct TELNET *tn;
 
-  tn = (struct TELNET *)malloc(sizeof(struct TELNET));
+  tn = (struct TELNET *)calloc(1, sizeof(struct TELNET));
   if(!tn)
     return CURLE_OUT_OF_MEMORY;
-  
-  conn->proto.telnet = (void *)tn; /* make us known */
 
-  memset(tn, 0, sizeof(struct TELNET));
+  conn->proto.telnet = (void *)tn; /* make us known */
 
   tn->telrcv_state = CURL_TS_DATA;
 
   /* Init suboptions */
   CURL_SB_CLEAR(tn);
 
-  /* Set all options to NO */
-#if 0
-  /* NO is zero => default fill pattern */
-  memset(tn->us, CURL_NO, 256);
-  memset(tn->usq, CURL_NO, 256);
-  memset(tn->us_preferred, CURL_NO, 256);
-  memset(tn->him, CURL_NO, 256);
-  memset(tn->himq, CURL_NO, 256);
-  memset(tn->him_preferred, CURL_NO, 256);
-#endif
   /* Set the options we want by default */
   tn->us_preferred[CURL_TELOPT_BINARY] = CURL_YES;
   tn->us_preferred[CURL_TELOPT_SGA] = CURL_YES;
@@ -248,31 +237,31 @@ static void negotiate(struct connectdata *conn)
 {
   int i;
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
-   
+
   for(i = 0;i < CURL_NTELOPTS;i++)
   {
     if(tn->us_preferred[i] == CURL_YES)
       set_local_option(conn, i, CURL_YES);
-      
+
     if(tn->him_preferred[i] == CURL_YES)
       set_remote_option(conn, i, CURL_YES);
   }
 }
 
 static void printoption(struct SessionHandle *data,
-			const char *direction, int cmd, int option)
+                        const char *direction, int cmd, int option)
 {
   const char *fmt;
   const char *opt;
-   
+
   if (data->set.verbose)
   {
     if (cmd == CURL_IAC)
     {
       if (CURL_TELCMD_OK(option))
-        Curl_infof(data, "%s IAC %s\n", direction, CURL_TELCMD(option));
+        infof(data, "%s IAC %s\n", direction, CURL_TELCMD(option));
       else
-        Curl_infof(data, "%s IAC %d\n", direction, option);
+        infof(data, "%s IAC %d\n", direction, option);
     }
     else
     {
@@ -288,12 +277,12 @@ static void printoption(struct SessionHandle *data,
           opt = NULL;
 
         if(opt)
-          Curl_infof(data, "%s %s %s\n", direction, fmt, opt);
+          infof(data, "%s %s %s\n", direction, fmt, opt);
         else
-          Curl_infof(data, "%s %s %d\n", direction, fmt, option);
+          infof(data, "%s %s %d\n", direction, fmt, option);
       }
       else
-        Curl_infof(data, "%s %d %d\n", direction, cmd, option);
+        infof(data, "%s %d %d\n", direction, cmd, option);
     }
   }
 }
@@ -301,13 +290,20 @@ static void printoption(struct SessionHandle *data,
 static void send_negotiation(struct connectdata *conn, int cmd, int option)
 {
    unsigned char buf[3];
+   ssize_t bytes_written;
+   int err;
+   struct SessionHandle *data = conn->data;
 
    buf[0] = CURL_IAC;
-   buf[1] = cmd;
-   buf[2] = option;
-   
-   (void)swrite(conn->sock[FIRSTSOCKET], buf, 3);
-   
+   buf[1] = (unsigned char)cmd;
+   buf[2] = (unsigned char)option;
+
+   bytes_written = swrite(conn->sock[FIRSTSOCKET], buf, 3);
+   if(bytes_written < 0) {
+     err = Curl_sockerrno();
+     failf(data,"Sending data failed (%d)",err);
+   }
+
    printoption(conn->data, "SENT", cmd, option);
 }
 
@@ -323,11 +319,11 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
         tn->him[option] = CURL_WANTYES;
         send_negotiation(conn, CURL_DO, option);
         break;
-	 
+
       case CURL_YES:
         /* Already enabled */
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->himq[option])
         {
@@ -340,7 +336,7 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->himq[option])
         {
@@ -361,12 +357,12 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
       case CURL_NO:
         /* Already disabled */
         break;
-	 
+
       case CURL_YES:
         tn->him[option] = CURL_WANTNO;
         send_negotiation(conn, CURL_DONT, option);
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->himq[option])
         {
@@ -378,7 +374,7 @@ void set_remote_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->himq[option])
         {
@@ -410,11 +406,11 @@ void rec_will(struct connectdata *conn, int option)
         send_negotiation(conn, CURL_DONT, option);
       }
       break;
-	 
+
     case CURL_YES:
       /* Already enabled */
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->himq[option])
       {
@@ -429,7 +425,7 @@ void rec_will(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->himq[option])
       {
@@ -445,7 +441,7 @@ void rec_will(struct connectdata *conn, int option)
       break;
   }
 }
-   
+
 static
 void rec_wont(struct connectdata *conn, int option)
 {
@@ -455,19 +451,19 @@ void rec_wont(struct connectdata *conn, int option)
     case CURL_NO:
       /* Already disabled */
       break;
-	 
+
     case CURL_YES:
       tn->him[option] = CURL_NO;
       send_negotiation(conn, CURL_DONT, option);
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->himq[option])
       {
         case CURL_EMPTY:
           tn->him[option] = CURL_NO;
           break;
-	 
+
         case CURL_OPPOSITE:
           tn->him[option] = CURL_WANTYES;
           tn->himq[option] = CURL_EMPTY;
@@ -475,7 +471,7 @@ void rec_wont(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->himq[option])
       {
@@ -490,7 +486,7 @@ void rec_wont(struct connectdata *conn, int option)
       break;
   }
 }
-   
+
 static void
 set_local_option(struct connectdata *conn, int option, int newstate)
 {
@@ -503,11 +499,11 @@ set_local_option(struct connectdata *conn, int option, int newstate)
         tn->us[option] = CURL_WANTYES;
         send_negotiation(conn, CURL_WILL, option);
         break;
-	 
+
       case CURL_YES:
         /* Already enabled */
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->usq[option])
         {
@@ -520,7 +516,7 @@ set_local_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->usq[option])
         {
@@ -541,12 +537,12 @@ set_local_option(struct connectdata *conn, int option, int newstate)
       case CURL_NO:
         /* Already disabled */
         break;
-	 
+
       case CURL_YES:
         tn->us[option] = CURL_WANTNO;
         send_negotiation(conn, CURL_WONT, option);
         break;
-	 
+
       case CURL_WANTNO:
         switch(tn->usq[option])
         {
@@ -558,7 +554,7 @@ set_local_option(struct connectdata *conn, int option, int newstate)
             break;
         }
         break;
-	 
+
       case CURL_WANTYES:
         switch(tn->usq[option])
         {
@@ -590,11 +586,11 @@ void rec_do(struct connectdata *conn, int option)
         send_negotiation(conn, CURL_WONT, option);
       }
       break;
-	 
+
     case CURL_YES:
       /* Already enabled */
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->usq[option])
       {
@@ -609,7 +605,7 @@ void rec_do(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->usq[option])
       {
@@ -626,7 +622,7 @@ void rec_do(struct connectdata *conn, int option)
   }
 }
 
-static   
+static
 void rec_dont(struct connectdata *conn, int option)
 {
   struct TELNET *tn = (struct TELNET *)conn->proto.telnet;
@@ -635,19 +631,19 @@ void rec_dont(struct connectdata *conn, int option)
     case CURL_NO:
       /* Already disabled */
       break;
-	 
+
     case CURL_YES:
       tn->us[option] = CURL_NO;
       send_negotiation(conn, CURL_WONT, option);
       break;
-	 
+
     case CURL_WANTNO:
       switch(tn->usq[option])
       {
         case CURL_EMPTY:
           tn->us[option] = CURL_NO;
           break;
-	 
+
         case CURL_OPPOSITE:
           tn->us[option] = CURL_WANTYES;
           tn->usq[option] = CURL_EMPTY;
@@ -655,7 +651,7 @@ void rec_dont(struct connectdata *conn, int option)
           break;
       }
       break;
-	 
+
     case CURL_WANTYES:
       switch(tn->usq[option])
       {
@@ -673,17 +669,17 @@ void rec_dont(struct connectdata *conn, int option)
 
 
 static void printsub(struct SessionHandle *data,
-		     int direction,		/* '<' or '>' */
-		     unsigned char *pointer,	/* where suboption data is */
-		     int length)		/* length of suboption data */
+                     int direction,             /* '<' or '>' */
+                     unsigned char *pointer,    /* where suboption data is */
+                     size_t length)             /* length of suboption data */
 {
-  int i = 0;
+  unsigned int i = 0;
 
   if (data->set.verbose)
   {
     if (direction)
     {
-      Curl_infof(data, "%s IAC SB ", (direction == '<')? "RCVD":"SENT");
+      infof(data, "%s IAC SB ", (direction == '<')? "RCVD":"SENT");
       if (length >= 3)
       {
         int j;
@@ -693,27 +689,27 @@ static void printsub(struct SessionHandle *data,
 
         if (i != CURL_IAC || j != CURL_SE)
         {
-          Curl_infof(data, "(terminated by ");
+          infof(data, "(terminated by ");
           if (CURL_TELOPT_OK(i))
-            Curl_infof(data, "%s ", CURL_TELOPT(i));
+            infof(data, "%s ", CURL_TELOPT(i));
           else if (CURL_TELCMD_OK(i))
-            Curl_infof(data, "%s ", CURL_TELCMD(i));
+            infof(data, "%s ", CURL_TELCMD(i));
           else
-            Curl_infof(data, "%d ", i);
+            infof(data, "%d ", i);
           if (CURL_TELOPT_OK(j))
-            Curl_infof(data, "%s", CURL_TELOPT(j));
+            infof(data, "%s", CURL_TELOPT(j));
           else if (CURL_TELCMD_OK(j))
-            Curl_infof(data, "%s", CURL_TELCMD(j));
+            infof(data, "%s", CURL_TELCMD(j));
           else
-            Curl_infof(data, "%d", j);
-          Curl_infof(data, ", not IAC SE!) ");
+            infof(data, "%d", j);
+          infof(data, ", not IAC SE!) ");
         }
       }
       length -= 2;
     }
     if (length < 1)
     {
-      Curl_infof(data, "(Empty suboption?)");
+      infof(data, "(Empty suboption?)");
       return;
     }
 
@@ -722,50 +718,50 @@ static void printsub(struct SessionHandle *data,
         case CURL_TELOPT_TTYPE:
         case CURL_TELOPT_XDISPLOC:
         case CURL_TELOPT_NEW_ENVIRON:
-          Curl_infof(data, "%s", CURL_TELOPT(pointer[0]));
+          infof(data, "%s", CURL_TELOPT(pointer[0]));
           break;
         default:
-          Curl_infof(data, "%s (unsupported)", CURL_TELOPT(pointer[0]));
+          infof(data, "%s (unsupported)", CURL_TELOPT(pointer[0]));
           break;
       }
     }
     else
-      Curl_infof(data, "%d (unknown)", pointer[i]);
+      infof(data, "%d (unknown)", pointer[i]);
 
     switch(pointer[1]) {
       case CURL_TELQUAL_IS:
-        Curl_infof(data, " IS");
+        infof(data, " IS");
         break;
       case CURL_TELQUAL_SEND:
-        Curl_infof(data, " SEND");
+        infof(data, " SEND");
         break;
       case CURL_TELQUAL_INFO:
-        Curl_infof(data, " INFO/REPLY");
+        infof(data, " INFO/REPLY");
         break;
       case CURL_TELQUAL_NAME:
-        Curl_infof(data, " NAME");
+        infof(data, " NAME");
         break;
     }
-      
+
     switch(pointer[0]) {
       case CURL_TELOPT_TTYPE:
       case CURL_TELOPT_XDISPLOC:
         pointer[length] = 0;
-        Curl_infof(data, " \"%s\"", &pointer[2]);
+        infof(data, " \"%s\"", &pointer[2]);
         break;
       case CURL_TELOPT_NEW_ENVIRON:
         if(pointer[1] == CURL_TELQUAL_IS) {
-          Curl_infof(data, " ");
+          infof(data, " ");
           for(i = 3;i < length;i++) {
             switch(pointer[i]) {
               case CURL_NEW_ENV_VAR:
-                Curl_infof(data, ", ");
+                infof(data, ", ");
                 break;
               case CURL_NEW_ENV_VALUE:
-                Curl_infof(data, " = ");
+                infof(data, " = ");
                 break;
               default:
-                Curl_infof(data, "%c", pointer[i]);
+                infof(data, "%c", pointer[i]);
                 break;
             }
           }
@@ -773,13 +769,13 @@ static void printsub(struct SessionHandle *data,
         break;
       default:
         for (i = 2; i < length; i++)
-          Curl_infof(data, " %.2x", pointer[i]);
+          infof(data, " %.2x", pointer[i]);
         break;
     }
-      
+
     if (direction)
     {
-      Curl_infof(data, "\n");
+      infof(data, "\n");
     }
   }
 }
@@ -855,8 +851,10 @@ static void suboption(struct connectdata *conn)
 {
   struct curl_slist *v;
   unsigned char temp[2048];
+  ssize_t bytes_written;
   size_t len;
   size_t tmplen;
+  int err;
   char varname[128];
   char varval[128];
   struct SessionHandle *data = conn->data;
@@ -869,7 +867,11 @@ static void suboption(struct connectdata *conn)
       snprintf((char *)temp, sizeof(temp),
                "%c%c%c%c%s%c%c", CURL_IAC, CURL_SB, CURL_TELOPT_TTYPE,
                CURL_TELQUAL_IS, tn->subopt_ttype, CURL_IAC, CURL_SE);
-      (void)swrite(conn->sock[FIRSTSOCKET], temp, len);
+      bytes_written = swrite(conn->sock[FIRSTSOCKET], temp, len);
+      if(bytes_written < 0) {
+        err = Curl_sockerrno();
+        failf(data,"Sending data failed (%d)",err);
+      }
       printsub(data, '>', &temp[2], len-2);
       break;
     case CURL_TELOPT_XDISPLOC:
@@ -877,7 +879,11 @@ static void suboption(struct connectdata *conn)
       snprintf((char *)temp, sizeof(temp),
                "%c%c%c%c%s%c%c", CURL_IAC, CURL_SB, CURL_TELOPT_XDISPLOC,
                CURL_TELQUAL_IS, tn->subopt_xdisploc, CURL_IAC, CURL_SE);
-      (void)swrite(conn->sock[FIRSTSOCKET], temp, len);
+      bytes_written = swrite(conn->sock[FIRSTSOCKET], temp, len);
+      if(bytes_written < 0) {
+        err = Curl_sockerrno();
+        failf(data,"Sending data failed (%d)",err);
+      }
       printsub(data, '>', &temp[2], len-2);
       break;
     case CURL_TELOPT_NEW_ENVIRON:
@@ -890,7 +896,7 @@ static void suboption(struct connectdata *conn)
         tmplen = (strlen(v->data) + 1);
         /* Add the variable only if it fits */
         if(len + tmplen < (int)sizeof(temp)-6) {
-          sscanf(v->data, "%127[^,],%s", varname, varval);
+          sscanf(v->data, "%127[^,],%127s", varname, varval);
           snprintf((char *)&temp[len], sizeof(temp) - len,
                    "%c%s%c%s", CURL_NEW_ENV_VAR, varname,
                    CURL_NEW_ENV_VALUE, varval);
@@ -900,7 +906,11 @@ static void suboption(struct connectdata *conn)
       snprintf((char *)&temp[len], sizeof(temp) - len,
                "%c%c", CURL_IAC, CURL_SE);
       len += 2;
-      (void)swrite(conn->sock[FIRSTSOCKET], temp, len);
+      bytes_written = swrite(conn->sock[FIRSTSOCKET], temp, len);
+      if(bytes_written < 0) {
+        err = Curl_sockerrno();
+        failf(data,"Sending data failed (%d)",err);
+      }
       printsub(data, '>', &temp[2], len-2);
       break;
   }
@@ -909,8 +919,8 @@ static void suboption(struct connectdata *conn)
 
 static
 void telrcv(struct connectdata *conn,
-            unsigned char *inbuf,	/* Data received from socket */
-            int count)			/* Number of bytes received */
+            unsigned char *inbuf,       /* Data received from socket */
+            ssize_t count)              /* Number of bytes received */
 {
   unsigned char c;
   int in = 0;
@@ -986,21 +996,21 @@ void telrcv(struct connectdata *conn,
         rec_will(conn, c);
         tn->telrcv_state = CURL_TS_DATA;
         continue;
-      
+
       case CURL_TS_WONT:
         printoption(data, "RCVD", CURL_WONT, c);
         tn->please_negotiate = 1;
         rec_wont(conn, c);
         tn->telrcv_state = CURL_TS_DATA;
         continue;
-      
+
       case CURL_TS_DO:
         printoption(data, "RCVD", CURL_DO, c);
         tn->please_negotiate = 1;
         rec_do(conn, c);
         tn->telrcv_state = CURL_TS_DATA;
         continue;
-      
+
       case CURL_TS_DONT:
         printoption(data, "RCVD", CURL_DONT, c);
         tn->please_negotiate = 1;
@@ -1025,22 +1035,19 @@ void telrcv(struct connectdata *conn,
           if (c != CURL_IAC)
           {
             /*
-             * This is an error.  We only expect to get
-             * "IAC IAC" or "IAC SE".  Several things may
-             * have happend.  An IAC was not doubled, the
-             * IAC SE was left off, or another option got
-             * inserted into the suboption are all possibilities.
-             * If we assume that the IAC was not doubled,
-             * and really the IAC SE was left off, we could
-             * get into an infinate loop here.  So, instead,
-             * we terminate the suboption, and process the
-             * partial suboption if we can.
+             * This is an error.  We only expect to get "IAC IAC" or "IAC SE".
+             * Several things may have happend.  An IAC was not doubled, the
+             * IAC SE was left off, or another option got inserted into the
+             * suboption are all possibilities.  If we assume that the IAC was
+             * not doubled, and really the IAC SE was left off, we could get
+             * into an infinate loop here.  So, instead, we terminate the
+             * suboption, and process the partial suboption if we can.
              */
-            CURL_SB_ACCUM(tn, (unsigned char)CURL_IAC);
+            CURL_SB_ACCUM(tn, CURL_IAC);
             CURL_SB_ACCUM(tn, c);
             tn->subpointer -= 2;
             CURL_SB_TERM(tn);
-	    
+
             printoption(data, "In SUBOPTION processing, RCVD", CURL_IAC, c);
             suboption(conn);   /* handle sub-option */
             tn->telrcv_state = CURL_TS_IAC;
@@ -1051,8 +1058,8 @@ void telrcv(struct connectdata *conn,
         }
         else
         {
-          CURL_SB_ACCUM(tn, (unsigned char)CURL_IAC);
-          CURL_SB_ACCUM(tn, (unsigned char)CURL_SE);
+          CURL_SB_ACCUM(tn, CURL_IAC);
+          CURL_SB_ACCUM(tn, CURL_SE);
           tn->subpointer -= 2;
           CURL_SB_TERM(tn);
           suboption(conn);   /* handle sub-option */
@@ -1076,7 +1083,7 @@ CURLcode Curl_telnet_done(struct connectdata *conn, CURLcode status)
   return CURLE_OK;
 }
 
-CURLcode Curl_telnet(struct connectdata *conn)
+CURLcode Curl_telnet(struct connectdata *conn, bool *done)
 {
   CURLcode code;
   struct SessionHandle *data = conn->data;
@@ -1090,20 +1097,22 @@ CURLcode Curl_telnet(struct connectdata *conn)
   WSAEVENT event_handle;
   WSANETWORKEVENTS events;
   HANDLE stdin_handle;
-  HANDLE objs[2];         
+  HANDLE objs[2];
   DWORD  obj_count;
   DWORD  wait_timeout;
   DWORD waitret;
   DWORD readfile_read;
 #else
-  fd_set readfd;
-  fd_set keepfd;
+  int interval_ms;
+  struct pollfd pfd[2];
 #endif
-  ssize_t nread;  
+  ssize_t nread;
   bool keepon = TRUE;
   char *buf = data->state.buffer;
   struct TELNET *tn;
 
+  *done = TRUE; /* uncontionally */
+
   code = init_telnet(conn);
   if(code)
     return code;
@@ -1135,7 +1144,7 @@ CURLcode Curl_telnet(struct connectdata *conn)
   create_event_func = GetProcAddress(wsock2,"WSACreateEvent");
   if (create_event_func == NULL) {
     failf(data,"failed to find WSACreateEvent function (%d)",
-	  GetLastError());
+          GetLastError());
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1144,7 +1153,7 @@ CURLcode Curl_telnet(struct connectdata *conn)
   close_event_func = GetProcAddress(wsock2,"WSACloseEvent");
   if (create_event_func == NULL) {
     failf(data,"failed to find WSACloseEvent function (%d)",
-	  GetLastError());
+          GetLastError());
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1153,7 +1162,7 @@ CURLcode Curl_telnet(struct connectdata *conn)
   event_select_func = GetProcAddress(wsock2,"WSAEventSelect");
   if (event_select_func == NULL) {
     failf(data,"failed to find WSAEventSelect function (%d)",
-	  GetLastError());
+          GetLastError());
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1162,7 +1171,7 @@ CURLcode Curl_telnet(struct connectdata *conn)
   enum_netevents_func = GetProcAddress(wsock2,"WSAEnumNetworkEvents");
   if (enum_netevents_func == NULL) {
     failf(data,"failed to find WSAEnumNetworkEvents function (%d)",
-	  GetLastError());
+          GetLastError());
     FreeLibrary(wsock2);
     return CURLE_FAILED_INIT;
   }
@@ -1193,19 +1202,19 @@ CURLcode Curl_telnet(struct connectdata *conn)
     FreeLibrary(wsock2);
     return 0;
   }
-                                   
-  /* If stdin_handle is a pipe, use PeekNamedPipe() method to check it, 
-     else use the old WaitForMultipleObjects() way */         
+
+  /* If stdin_handle is a pipe, use PeekNamedPipe() method to check it,
+     else use the old WaitForMultipleObjects() way */
   if(GetFileType(stdin_handle) == FILE_TYPE_PIPE) {
     /* Don't wait for stdin_handle, just wait for event_handle */
-    obj_count = 1;      
+    obj_count = 1;
     /* Check stdin_handle per 100 milliseconds */
-    wait_timeout = 100;  
+    wait_timeout = 100;
   } else {
     obj_count = 2;
     wait_timeout = INFINITE;
   }
-  
+
   /* Keep on listening and act on events */
   while(keepon) {
     waitret = WaitForMultipleObjects(obj_count, objs, FALSE, wait_timeout);
@@ -1217,72 +1226,73 @@ CURLcode Curl_telnet(struct connectdata *conn)
       ssize_t bytes_written;
       char *buffer = buf;
 
-      for(;;) {
-        if(!PeekNamedPipe(stdin_handle,NULL,0,NULL,&nread,NULL)) {
+      while(1) {
+        if(!PeekNamedPipe(stdin_handle, NULL, 0, NULL, &readfile_read, NULL)) {
           keepon = FALSE;
           break;
         }
-        
+        nread = readfile_read;
+
         if(!nread)
           break;
-          
+
         if(!ReadFile(stdin_handle, buf, sizeof(data->state.buffer),
                      &readfile_read, NULL)) {
           keepon = FALSE;
           break;
         }
         nread = readfile_read;
-        
+
         while(nread--) {
           outbuf[0] = *buffer++;
           out_count = 1;
           if(outbuf[0] == CURL_IAC)
             outbuf[out_count++] = CURL_IAC;
-            
+
           Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
                      out_count, &bytes_written);
         }
       }
-    }      
+    }
     break;
-    
+
     case WAIT_OBJECT_0 + 1:
     {
       unsigned char outbuf[2];
       int out_count = 0;
       ssize_t bytes_written;
       char *buffer = buf;
-              
+
       if(!ReadFile(stdin_handle, buf, sizeof(data->state.buffer),
                    &readfile_read, NULL)) {
         keepon = FALSE;
         break;
       }
       nread = readfile_read;
-        
+
       while(nread--) {
         outbuf[0] = *buffer++;
         out_count = 1;
         if(outbuf[0] == CURL_IAC)
           outbuf[out_count++] = CURL_IAC;
-          
+
         Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
                    out_count, &bytes_written);
       }
     }
     break;
-      
+
     case WAIT_OBJECT_0:
       if(enum_netevents_func(sockfd, event_handle, &events)
          != SOCKET_ERROR) {
         if(events.lNetworkEvents & FD_READ) {
           /* This reallu OUGHT to check its return code. */
           (void)Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
-            
+
           telrcv(conn, (unsigned char *)buf, nread);
-          
+
           fflush(stdout);
-            
+
           /* Negotiate if the peer has started negotiating,
              otherwise don't. We don't want to speak telnet with
              non-telnet servers, like POP or SMTP. */
@@ -1291,7 +1301,7 @@ CURLcode Curl_telnet(struct connectdata *conn)
             tn->already_negotiated = 1;
           }
         }
-          
+
         if(events.lNetworkEvents & FD_CLOSE) {
           keepon = FALSE;
         }
@@ -1315,32 +1325,26 @@ CURLcode Curl_telnet(struct connectdata *conn)
   if (!FreeLibrary(wsock2))
     infof(data,"FreeLibrary(wsock2) failed (%d)",GetLastError());
 #else
-  FD_ZERO (&readfd);		/* clear it */
-  FD_SET (sockfd, &readfd);
-  FD_SET (0, &readfd);
-
-  keepfd = readfd;
+  pfd[0].fd = sockfd;
+  pfd[0].events = POLLIN;
+  pfd[1].fd = 0;
+  pfd[1].events = POLLIN;
+  interval_ms = 1 * 1000;
 
   while (keepon) {
-    struct timeval interval;
-
-    readfd = keepfd;		/* set this every lap in the loop */
-    interval.tv_sec = 1;
-    interval.tv_usec = 0;
-
-    switch (select (sockfd + 1, &readfd, NULL, NULL, &interval)) {
-    case -1:			/* error, stop reading */
+    switch (Curl_poll(pfd, 2, interval_ms)) {
+    case -1:                    /* error, stop reading */
       keepon = FALSE;
       continue;
-    case 0:			/* timeout */
+    case 0:                     /* timeout */
       break;
-    default:			/* read! */
-      if(FD_ISSET(0, &readfd)) { /* read from stdin */
+    default:                    /* read! */
+      if(pfd[1].revents & POLLIN) { /* read from stdin */
         unsigned char outbuf[2];
         int out_count = 0;
         ssize_t bytes_written;
         char *buffer = buf;
-        
+
         nread = read(0, buf, 255);
 
         while(nread--) {
@@ -1348,13 +1352,13 @@ CURLcode Curl_telnet(struct connectdata *conn)
           out_count = 1;
           if(outbuf[0] == CURL_IAC)
             outbuf[out_count++] = CURL_IAC;
-      
+
           Curl_write(conn, conn->sock[FIRSTSOCKET], outbuf,
                      out_count, &bytes_written);
         }
       }
 
-      if(FD_ISSET(sockfd, &readfd)) {
+      if(pfd[0].revents & POLLIN) {
         /* This OUGHT to check the return code... */
         (void)Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
 
