@@ -1,18 +1,18 @@
 #ifndef __URLGLOB_H
 #define __URLGLOB_H
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2005, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -20,9 +20,13 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: urlglob.h,v 1.12 2003/06/10 09:42:22 bagder Exp $
+ * $Id: urlglob.h,v 1.17 2005/11/10 22:11:02 bagder Exp $
  ***************************************************************************/
-typedef enum {UPTSet=1,UPTCharRange,UPTNumRange} URLPatternType;
+typedef enum {
+  UPTSet=1,
+  UPTCharRange,
+  UPTNumRange
+} URLPatternType;
 
 typedef struct {
   URLPatternType type;
@@ -35,11 +39,13 @@ typedef struct {
     struct {
       char min_c, max_c;
       char ptr_c;
+      int step;
     } CharRange;
     struct {
       int min_n, max_n;
       short padlength;
       int ptr_n;
+      int step;
     } NumRange ;
   } content;
 } URLPattern;
@@ -47,15 +53,16 @@ typedef struct {
 typedef struct {
   char* literal[10];
   URLPattern pattern[9];
-  int size;
-  int urllen;
+  size_t size;
+  size_t urllen;
   char *glob_buffer;
   char beenhere;
+  char errormsg[80]; /* error message buffer */
 } URLGlob;
 
 int glob_url(URLGlob**, char*, int *, FILE *);
 char* glob_next_url(URLGlob*);
-char* glob_match_url(char*, URLGlob *); 
+char* glob_match_url(char*, URLGlob *);
 void glob_cleanup(URLGlob* glob);
 
 #endif
