@@ -6,7 +6,7 @@
 #                            | (__| |_| |  _ <| |___
 #                             \___|\___/|_| \_\_____|
 #
-# Copyright (C) 1998 - 2005, Daniel Stenberg, <daniel@haxx.se>, et al.
+# Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: testcurl.pl,v 1.38 2005/12/13 19:07:33 danf Exp $
+# $Id: testcurl.pl,v 1.40 2006-05-27 11:36:05 bagder Exp $
 ###########################################################################
 
 ###########################
@@ -68,7 +68,7 @@ use vars qw($name $email $desc $confopts $runtestopts $setupfile $mktarball
             $nocvsup $nobuildconf $crosscompile);
 
 # version of this script
-$version='$Revision: 1.38 $';
+$version='$Revision: 1.40 $';
 $fixed=0;
 
 # Determine if we're running from CVS or a canned copy of curl,
@@ -129,10 +129,9 @@ if ($^O eq 'MSWin32' || $targetos) {
     # If no target defined on Win32 lets assume vc
     $targetos = 'vc';
   }
-  if ($targetos =~ /vc/ || $targetos =~ /mingw32/ || $targetos =~ /borland/) {
+  if ($targetos =~ /vc/ || $targetos =~ /borland/) {
     $binext = '.exe';
-    $libext = '.lib' if ($targetos =~ /vc/ || $targetos =~ /borland/);
-    $libext = '.a' if ($targetos =~ /mingw32/);
+    $libext = '.lib';
   }
   elsif ($targetos =~ /netware/) {
     $configurebuild = 0;
@@ -141,7 +140,8 @@ if ($^O eq 'MSWin32' || $targetos) {
   }
 }
 
-if (($^O eq 'MSWin32') && ($targetos =~ /vc/ || $targetos =~ /mingw32/ || $targetos =~ /borland/)) {
+if (($^O eq 'MSWin32') &&
+    ($targetos =~ /vc/ || $targetos =~ /mingw32/ || $targetos =~ /borland/)) {
 
   # Set these things only when building ON Windows and for Win32 platform.
   # FOR Windows since we might be cross-compiling on another system. Non-
@@ -536,17 +536,17 @@ else {
 }
 
 if (-f "lib/libcurl$libext") {
-  logit "lib/libcurl was created fine (libcurl$libext)";
+  logit "libcurl was created fine (libcurl$libext)";
 }
 else {
-  logit "lib/libcurl was not created (libcurl$libext)";
+  mydie "libcurl was not created (libcurl$libext)";
 }
 
 if (-f "src/curl$binext") {
-  logit "src/curl was created fine (curl$binext)";
+  logit "curl was created fine (curl$binext)";
 }
 else {
-  mydie "src/curl was not created (curl$binext)";
+  mydie "curl was not created (curl$binext)";
 }
 
 if ($targetos =~ /netware/) {
